==> apache-skywalking-go-0.5.0-src.tgz: apache-skywalking-go-0.5.0/plugins/http/client_intercepter.go <==
// Licensed to Apache Software Foundation (ASF) under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Apache Software Foundation (ASF) licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package http

import (
	"fmt"
	"net/http"

	"github.com/apache/skywalking-go/plugins/core/operator"
	"github.com/apache/skywalking-go/plugins/core/tracing"
)

type ClientInterceptor struct {
}

func (h *ClientInterceptor) BeforeInvoke(invocation operator.Invocation) error {
	request := invocation.Args()[0].(*http.Request)
	s, err := tracing.CreateExitSpan(fmt.Sprintf("%s:%s", request.Method, request.URL.Path), request.Host, func(headerKey, headerValue string) error {
		request.Header.Add(headerKey, headerValue)
		return nil
	}, tracing.WithLayer(tracing.SpanLayerHTTP),
		tracing.WithTag(tracing.TagHTTPMethod, request.Method),
		tracing.WithTag(tracing.TagURL, request.Host+request.URL.Path),
		tracing.WithComponent(5005))
	if err != nil {
		return err
	}
	invocation.SetContext(s)
	return nil
}

func (h *ClientInterceptor) AfterInvoke(invocation operator.Invocation, result ...interface{}) error {
	if invocation.GetContext() == nil {
		return nil
	}
	span := invocation.GetContext().(tracing.Span)
	if resp, ok := result[0].(*http.Response); ok && resp != nil {
		span.Tag(tracing.TagStatusCode, fmt.Sprintf("%d", resp.StatusCode))
	}
	if err, ok := result[1].(error); ok && err != nil {
		span.Error(err.Error())
	}
	span.End()
	return nil
}

==> apache-skywalking-go-0.6.0-src.tgz: apache-skywalking-go-0.6.0/plugins/http/client_intercepter.go <==
// Licensed to Apache Software Foundation (ASF) under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Apache Software Foundation (ASF) licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package http

import (
	"fmt"
	"net/http"

	"github.com/apache/skywalking-go/plugins/core/operator"
	"github.com/apache/skywalking-go/plugins/core/tracing"
)

type ClientInterceptor struct {
}

func (h *ClientInterceptor) BeforeInvoke(invocation operator.Invocation) error {
	request := invocation.Args()[0].(*http.Request)
	host := request.Host
	if host == "" && request.URL != nil {
		host = request.URL.Host
	}
	s, err := tracing.CreateExitSpan(fmt.Sprintf("%s:%s", request.Method, request.URL.Path), host, func(headerKey, headerValue string) error {
		request.Header.Add(headerKey, headerValue)
		return nil
	}, tracing.WithLayer(tracing.SpanLayerHTTP),
		tracing.WithTag(tracing.TagHTTPMethod, request.Method),
		tracing.WithTag(tracing.TagURL, host+request.URL.Path),
		tracing.WithComponent(5005))
	if err != nil {
		return err
	}
	invocation.SetContext(s)
	return nil
}

func (h *ClientInterceptor) AfterInvoke(invocation operator.Invocation, result ...interface{}) error {
	if invocation.GetContext() == nil {
		return nil
	}
	span := invocation.GetContext().(tracing.Span)
	if resp, ok := result[0].(*http.Response); ok && resp != nil {
		span.Tag(tracing.TagStatusCode, fmt.Sprintf("%d", resp.StatusCode))
		if resp.StatusCode >= 400 {
			span.ErrorOccured()
		}
	}
	if err, ok := result[1].(error); ok && err != nil {
		span.Error(err.Error())
	}
	span.End()
	return nil
}
